from pygame import Surface
from pygame.image import load
from pygame.draw import circle
from pygame.locals import *

from dark_stew.pgfw.GameChild import GameChild

class Floor(GameChild, Surface):

    def __init__(self, parent):
        GameChild.__init__(self, parent)
        self.display_surface = self.get_display_surface()
        self.input = self.get_input()
        self.load_configuration()
        self.init_surface()
        self.paint()
        self.add_film()

    def load_configuration(self):
        config = self.get_configuration("floor")
        self.tile_path = self.get_resource("floor", "tile-path")
        self.film_path = self.get_resource("floor", "film-path")
        self.film_opacity = config["film-opacity"]

    def init_surface(self):
        Surface.__init__(self, self.parent.size)

    def paint(self):
        tile = load(self.tile_path)
        self.fill_tile(tile)

    def fill_tile(self, tile, blend=0):
        for x in xrange(0, self.get_width(), tile.get_width()):
            for y in xrange(0, self.get_height(), tile.get_height()):
                self.blit(tile, (x, y), None, blend)

    def add_film(self):
        tile = load(self.film_path).convert_alpha()
        tile.set_alpha(self.film_opacity)
        self.fill_tile(tile, BLEND_ADD)

    def add_holes(self):
        for pool in self.parent.pools:
            self.fill(self.transparent_color, pool)

    def draw(self):
        surface = self.display_surface
        for corner in self.parent.get_corners():
            surface.blit(self, corner)
